package com.ruoyi.demo.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.ruoyi.demo.entity.Statistics;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

@Mapper
public interface StatisticsMapper extends BaseMapper<Statistics> {
    Integer selectTotal();
    // 累计用户
    List<String> selectUserIdsByEquipmentCode(@Param("equipmentCode") String equipmentCode);
    // 累计扫码
    Integer selectScanByEquipmentCode(@Param("equipmentCode") String equipmentCode);
    // 累计交易
    Integer selectDealCountByEquipmentCode(@Param("equipmentCode") String equipmentCode);
    // 累计收入
    Float selectTotalPriceByEquipmentCode(@Param("equipmentCode") String equipmentCode);

    // 扫码记录
    // 按月分析
    Integer selectTimeDisByMonth(@Param("month") String month);
    // 按日分析
    Integer selectTimeDisByDay(@Param("day") String day);
    // 按年分析
    Integer selectTimeDisByYear(@Param("year") String year);
}
